@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21305 $
+ * $Revision: 21695 $
  * $Author: rjongbloed $
- * $Date: 2008-10-13 07:02:42 +0000 (Mon, 13 Oct 2008) $
+ * $Date: 2008-12-04 02:01:41 +0000 (Thu, 04 Dec 2008) $
  */
 
 //////////////////////////////////////////////////
@@ -77,11 +77,12 @@ PBoolean PInterfaceMonitorClient::GetInterfaceInfo(const PString & iface, Interf
 
 //////////////////////////////////////////////////
 
-PInterfaceMonitor::PInterfaceMonitor(unsigned refresh, bool _runMonitorThread)
-  : runMonitorThread(_runMonitorThread)
-  , refreshInterval(refresh)
-  , updateThread(NULL)
-  , interfaceFilter(NULL)
+PInterfaceMonitor::PInterfaceMonitor(unsigned refresh, bool runMonitorThread)
+  : m_runMonitorThread(runMonitorThread)
+  , m_refreshInterval(refresh)
+  , m_updateThread(NULL)
+  , m_threadRunning(false)
+  , m_interfaceFilter(NULL)
 {
 }
 
@@ -90,7 +91,7 @@ PInterfaceMonitor::~PInterfaceMonitor()
 {
   Stop();
 
-  delete interfaceFilter;
+  delete m_interfaceFilter;
 }
 
 
@@ -100,54 +101,57 @@ PInterfaceMonitor & PInterfaceMonitor::GetInstance()
 }
 
 
-void PInterfaceMonitor::SetRefreshInterval (unsigned refresh)
+void PInterfaceMonitor::SetRefreshInterval(unsigned refresh)
 {
-  refreshInterval = refresh;
+  m_refreshInterval = refresh;
 }
 
 
-void PInterfaceMonitor::SetRunMonitorThread(bool _runMonitorThread)
+void PInterfaceMonitor::SetRunMonitorThread(bool runMonitorThread)
 {
-  runMonitorThread = _runMonitorThread;
+  m_runMonitorThread = runMonitorThread;
 }
 
 
-bool PInterfaceMonitor::Start()
+void PInterfaceMonitor::Start()
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal guard(m_threadMutex);
   
-  if (runMonitorThread && updateThread != NULL)
-    return false; // Already running
-    
-  PIPSocket::GetInterfaceTable(currentInterfaces);
-  PTRACE(4, "IfaceMon\tInitial interface list:\n" << setfill('\n') << currentInterfaces << setfill(' '));
+  if (m_updateThread != NULL) // Already running
+    m_signalUpdate.Signal();
+  else {
+    m_interfacesMutex.Wait();
+    PIPSocket::GetInterfaceTable(m_interfaces);
+    PTRACE(4, "IfaceMon\tInitial interface list:\n" << setfill('\n') << m_interfaces << setfill(' '));
+    m_interfacesMutex.Signal();
 
-  if (runMonitorThread) {
-    updateThread = new PThreadObj<PInterfaceMonitor>(*this, &PInterfaceMonitor::UpdateThreadMain);
-    updateThread->SetThreadName("Network Interface Monitor");
+    if (m_runMonitorThread) {
+      m_threadRunning = true;
+      m_updateThread = new PThreadObj<PInterfaceMonitor>(*this, &PInterfaceMonitor::UpdateThreadMain);
+      m_updateThread->SetThreadName("Network Interface Monitor");
+    }
   }
-  return true;
-  
 }
 
 
 void PInterfaceMonitor::Stop()
 {
-  mutex.Wait();
+  m_threadMutex.Wait();
 
   // shutdown the update thread
-  if (updateThread != NULL) {
-    threadRunning.Signal();
+  if (m_updateThread != NULL) {
+    m_threadRunning = false;
+    m_signalUpdate.Signal();
 
-    mutex.Signal();
-    updateThread->WaitForTermination();
-    mutex.Wait();
+    m_threadMutex.Signal();
+    m_updateThread->WaitForTermination();
+    m_threadMutex.Wait();
 
-    delete updateThread;
-    updateThread = NULL;
+    delete m_updateThread;
+    m_updateThread = NULL;
   }
 
-  mutex.Signal();
+  m_threadMutex.Signal();
 }
 
 
@@ -204,39 +208,43 @@ void PInterfaceMonitor::RefreshInterfaceList()
   PIPSocket::InterfaceTable newInterfaces;
   PIPSocket::GetInterfaceTable(newInterfaces);
 
-  PWaitAndSignal m(mutex);
+  m_interfacesMutex.Wait();
 
   // if changed, then update the internal list
-  if (!CompareInterfaceLists(currentInterfaces, newInterfaces)) {
-
-    PIPSocket::InterfaceTable oldInterfaces = currentInterfaces;
-    currentInterfaces = newInterfaces;
-    
-    PTRACE(4, "IfaceMon\tInterface change detected, new list:\n" << setfill('\n') << currentInterfaces << setfill(' '));
-    
-    // calculate the set of interfaces to add / remove beforehand
-    PIPSocket::InterfaceTable interfacesToAdd;
-    PIPSocket::InterfaceTable interfacesToRemove;
-    interfacesToAdd.DisallowDeleteObjects();
-    interfacesToRemove.DisallowDeleteObjects();
-    
-    PINDEX i;
-    // look for interfaces to add that are in new list that are not in the old list
-    for (i = 0; i < newInterfaces.GetSize(); ++i) {
-      PIPSocket::InterfaceEntry & newEntry = newInterfaces[i];
-      if (!newEntry.GetAddress().IsLoopback() && !IsInterfaceInList(newEntry, oldInterfaces))
-        interfacesToAdd.Append(&newEntry);
-    }
-    // look for interfaces to remove that are in old list that are not in the new list
-    for (i = 0; i < oldInterfaces.GetSize(); ++i) {
-      PIPSocket::InterfaceEntry & oldEntry = oldInterfaces[i];
-      if (!oldEntry.GetAddress().IsLoopback() && !IsInterfaceInList(oldEntry, newInterfaces))
-        interfacesToRemove.Append(&oldEntry);
-    }
-
-    PIPSocket::ClearNameCache();
-    OnInterfacesChanged(interfacesToAdd, interfacesToRemove);
+  if (CompareInterfaceLists(m_interfaces, newInterfaces)) {
+    m_interfacesMutex.Signal();
+    return;
   }
+
+  PIPSocket::InterfaceTable oldInterfaces = m_interfaces;
+  m_interfaces = newInterfaces;
+
+  PTRACE(4, "IfaceMon\tInterface change detected, new list:\n" << setfill('\n') << newInterfaces << setfill(' '));
+  
+  m_interfacesMutex.Signal();
+
+  // calculate the set of interfaces to add / remove beforehand
+  PIPSocket::InterfaceTable interfacesToAdd;
+  PIPSocket::InterfaceTable interfacesToRemove;
+  interfacesToAdd.DisallowDeleteObjects();
+  interfacesToRemove.DisallowDeleteObjects();
+  
+  PINDEX i;
+  // look for interfaces to add that are in new list that are not in the old list
+  for (i = 0; i < newInterfaces.GetSize(); ++i) {
+    PIPSocket::InterfaceEntry & newEntry = newInterfaces[i];
+    if (!newEntry.GetAddress().IsLoopback() && !IsInterfaceInList(newEntry, oldInterfaces))
+      interfacesToAdd.Append(&newEntry);
+  }
+  // look for interfaces to remove that are in old list that are not in the new list
+  for (i = 0; i < oldInterfaces.GetSize(); ++i) {
+    PIPSocket::InterfaceEntry & oldEntry = oldInterfaces[i];
+    if (!oldEntry.GetAddress().IsLoopback() && !IsInterfaceInList(oldEntry, newInterfaces))
+      interfacesToRemove.Append(&oldEntry);
+  }
+
+  PIPSocket::ClearNameCache();
+  OnInterfacesChanged(interfacesToAdd, interfacesToRemove);
 }
 
 
@@ -245,9 +253,10 @@ void PInterfaceMonitor::UpdateThreadMain()
   PTRACE(4, "IfaceMon\tStarted interface monitor thread.");
 
   // check for interface changes periodically
-  do {
+  while (m_threadRunning) {
     RefreshInterfaceList();
-  } while (!threadRunning.Wait(refreshInterval));
+    PIPSocket::WaitForRouteTableChange(m_refreshInterval, &m_signalUpdate);
+  }
 
   PTRACE(4, "IfaceMon\tFinished interface monitor thread.");
 }
@@ -303,13 +312,12 @@ static PBoolean InterfaceMatches(const PIPSocket::Address & addr,
 PStringArray PInterfaceMonitor::GetInterfaces(bool includeLoopBack, 
                                               const PIPSocket::Address & destination)
 {
-  PWaitAndSignal guard(mutex);
+  PWaitAndSignal guard(m_interfacesMutex);
   
-  PIPSocket::InterfaceTable ifaces = currentInterfaces;
+  PIPSocket::InterfaceTable ifaces = m_interfaces;
   
-  if (interfaceFilter != NULL && !destination.IsAny()) {
-    ifaces = interfaceFilter->FilterInterfaces(destination, ifaces);
-  }
+  if (m_interfaceFilter != NULL && !destination.IsAny())
+    ifaces = m_interfaceFilter->FilterInterfaces(destination, ifaces);
 
   PStringArray names;
 
@@ -331,13 +339,13 @@ PStringArray PInterfaceMonitor::GetInterfaces(bool includeLoopBack,
 bool PInterfaceMonitor::IsValidBindingForDestination(const PIPSocket::Address & binding,
                                                      const PIPSocket::Address & destination)
 {
-  PWaitAndSignal guard(mutex);
+  PWaitAndSignal guard(m_interfacesMutex);
   
-  if (interfaceFilter == NULL)
+  if (m_interfaceFilter == NULL)
     return true;
   
-  PIPSocket::InterfaceTable ifaces = currentInterfaces;
-  ifaces = interfaceFilter->FilterInterfaces(destination, ifaces);
+  PIPSocket::InterfaceTable ifaces = m_interfaces;
+  ifaces = m_interfaceFilter->FilterInterfaces(destination, ifaces);
   for (PINDEX i = 0; i < ifaces.GetSize(); i++) {
     if (ifaces[i].GetAddress() == binding)
       return true;
@@ -353,10 +361,10 @@ bool PInterfaceMonitor::GetInterfaceInfo(const PString & iface, PIPSocket::Inter
   if (!SplitInterfaceDescription(iface, addr, name))
     return false;
 
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal guard(m_interfacesMutex);
 
-  for (PINDEX i = 0; i < currentInterfaces.GetSize(); ++i) {
-    PIPSocket::InterfaceEntry & entry = currentInterfaces[i];
+  for (PINDEX i = 0; i < m_interfaces.GetSize(); ++i) {
+    PIPSocket::InterfaceEntry & entry = m_interfaces[i];
     if (InterfaceMatches(addr, name, entry)) {
       info = entry;
       return true;
@@ -380,38 +388,40 @@ bool PInterfaceMonitor::IsMatchingInterface(const PString & iface, const PIPSock
 
 void PInterfaceMonitor::SetInterfaceFilter(PInterfaceFilter * filter)
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal guard(m_interfacesMutex);
   
-  delete interfaceFilter;
-  interfaceFilter = filter;
+  delete m_interfaceFilter;
+  m_interfaceFilter = filter;
 }
 
 
 void PInterfaceMonitor::AddClient(PInterfaceMonitorClient * client)
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal guard(m_clientsMutex);
 
-  if (currentClients.empty()) {
+  if (m_clients.empty()) {
     Start();
-    currentClients.push_back(client);
-  } else {
-    for (ClientList_T::iterator iter = currentClients.begin(); iter != currentClients.end(); ++iter) {
+    m_clients.push_back(client);
+  }
+  else {
+    for (ClientList_T::iterator iter = m_clients.begin(); iter != m_clients.end(); ++iter) {
       if ((*iter)->GetPriority() >= client->GetPriority()) {
-        currentClients.insert(iter, client);
+        m_clients.insert(iter, client);
         return;
       }
     }
-    currentClients.push_back(client);
+    m_clients.push_back(client);
   }
 }
 
 
 void PInterfaceMonitor::RemoveClient(PInterfaceMonitorClient * client)
 {
-  mutex.Wait();
-  currentClients.remove(client);
-  bool stop = currentClients.empty();
-  mutex.Signal();
+  m_clientsMutex.Wait();
+  m_clients.remove(client);
+  bool stop = m_clients.empty();
+  m_clientsMutex.Signal();
+
   if (stop)
     Stop();
 }
@@ -419,9 +429,9 @@ void PInterfaceMonitor::RemoveClient(PInterfaceMonitorClient * client)
 void PInterfaceMonitor::OnInterfacesChanged(const PIPSocket::InterfaceTable & addedInterfaces,
                                             const PIPSocket::InterfaceTable & removedInterfaces)
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal guard(m_clientsMutex);
   
-  for (ClientList_T::reverse_iterator iter = currentClients.rbegin(); iter != currentClients.rend(); ++iter) {
+  for (ClientList_T::reverse_iterator iter = m_clients.rbegin(); iter != m_clients.rend(); ++iter) {
     PInterfaceMonitorClient * client = *iter;
     if (client->LockReadWrite()) {
       for (PINDEX i = 0; i < addedInterfaces.GetSize(); i++)
@@ -436,9 +446,9 @@ void PInterfaceMonitor::OnInterfacesChanged(const PIPSocket::InterfaceTable & ad
 
 void PInterfaceMonitor::OnRemoveNatMethod(const PNatMethod  * natMethod)
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal guard(m_clientsMutex);
   
-  for (ClientList_T::reverse_iterator iter = currentClients.rbegin(); iter != currentClients.rend(); ++iter) {
+  for (ClientList_T::reverse_iterator iter = m_clients.rbegin(); iter != m_clients.rend(); ++iter) {
     PInterfaceMonitorClient *client = *iter;
     if (client->LockReadWrite()) {
       client->OnRemoveNatMethod(natMethod);
@@ -464,7 +474,7 @@ bool PMonitoredSockets::CreateSocket(SocketInfo & info, const PIPSocket::Address
   delete info.socket;
   info.socket = NULL;
   
-  if (natMethod != NULL) {
+  if (natMethod != NULL && natMethod->IsAvailable(binding)) {
     PIPSocket::Address address;
     WORD port;
     natMethod->GetServerAddress(address, port);
@@ -670,7 +680,7 @@ PChannel::Errors PMonitoredSockets::ReadFromSocket(SocketInfo & info,
 
 PMonitoredSockets * PMonitoredSockets::Create(const PString & iface, bool reuseAddr, PNatMethod * natMethod)
 {
-  if (iface.IsEmpty() || iface == "*" || PIPSocket::Address(iface).IsAny())
+  if (iface.IsEmpty() || iface == "*" || (iface[0] != '%' && PIPSocket::Address(iface).IsAny()))
     return new PMonitoredSocketBundle(reuseAddr, natMethod);
   else
     return new PSingleMonitoredSocket(iface, reuseAddr, natMethod);
@@ -1053,8 +1063,10 @@ PBoolean PSingleMonitoredSocket::Open(WORD port)
 {
   PSafeLockReadWrite guard(*this);
 
-  if (opened && theInfo.socket != NULL && theInfo.socket->IsOpen())
-    return false;
+  if (opened && localPort == port && theInfo.socket != NULL && theInfo.socket->IsOpen())
+    return true;
+
+  Close();
 
   opened = true;
 
@@ -1063,8 +1075,10 @@ PBoolean PSingleMonitoredSocket::Open(WORD port)
   if (theEntry.GetAddress().IsAny())
     GetInterfaceInfo(theInterface, theEntry);
 
-  if (theEntry.GetAddress().IsAny())
-    return true;
+  if (theEntry.GetAddress().IsAny()) {
+    PTRACE(3, "MonSock\tNot creating socket as interface \"" << theEntry.GetName() << "\" is  not up.");
+    return true; // Still say successful though
+  }
 
   if (!CreateSocket(theInfo, theEntry.GetAddress()))
     return false;
@@ -1078,6 +1092,9 @@ PBoolean PSingleMonitoredSocket::Close()
 {
   PSafeLockReadWrite guard(*this);
 
+  if (!opened)
+    return true;
+
   opened = false;
   interfaceAddedSignal.Close(); // Fail safe break out of Select()
   return DestroySocket(theInfo);
@@ -1148,7 +1165,7 @@ void PSingleMonitoredSocket::OnAddInterface(const InterfaceEntry & entry)
   if (!SplitInterfaceDescription(theInterface, addr, name))
     return;
 
-  if (entry.GetAddress() == addr && entry.GetName().NumCompare(name) == EqualTo) {
+  if ((!addr.IsValid() || entry.GetAddress() == addr) && entry.GetName().NumCompare(name) == EqualTo) {
     theEntry = entry;
     if (!Open(localPort))
       theEntry = InterfaceEntry();
