@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21361 $
+ * $Revision: 21400 $
  * $Author: csoutheren $
- * $Date: 2008-10-16 04:47:14 +0000 (Thu, 16 Oct 2008) $
+ * $Date: 2008-10-21 08:57:27 +0000 (Tue, 21 Oct 2008) $
  */
 
 #ifdef __GNUC__
@@ -59,6 +59,8 @@ static const char DefaultYUVFileName[] = "*.yuv";
 class PVideoInputDevice_YUVFile_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
 {
   public:
+    typedef PFactory<PVideoFile> FileTypeFactory_T;
+
     virtual PObject * CreateInstance(int /*userData*/) const
     {
       return new PVideoInputDevice_YUVFile;
@@ -70,13 +72,23 @@ class PVideoInputDevice_YUVFile_PluginServiceDescriptor : public PDevicePluginSe
     virtual bool ValidateDeviceName(const PString & deviceName, int /*userData*/) const
     {
       PCaselessString adjustedDevice = deviceName;
-      PINDEX length = adjustedDevice.GetLength();
-      if (length > 5 && adjustedDevice.NumCompare(".yuv*", 5, length-5) == PObject::EqualTo)
-        adjustedDevice.Delete(length-1, 1);
-      else if (length < 5 || adjustedDevice.NumCompare(".yuv", 4, length-4) != PObject::EqualTo)
-        return false;
 
-      return PFile::Access(adjustedDevice, PFile::ReadOnly);
+      FileTypeFactory_T::KeyList_T keyList = FileTypeFactory_T::GetKeyList();
+      FileTypeFactory_T::KeyList_T::iterator r;
+      for (r = keyList.begin(); r != keyList.end(); ++r) {
+        PString ext = *r;
+        PINDEX extLen = ext.GetLength();
+        PINDEX length = adjustedDevice.GetLength();
+        if (length > (2+extLen) && adjustedDevice.NumCompare(PString(".") + ext + "*", 2+extLen, length-(2+extLen)) == PObject::EqualTo)
+          adjustedDevice.Delete(length-1, 1);
+        else if (length < (2+extLen) || adjustedDevice.NumCompare(PString(".") + ext, 1+extLen, length-(1+extLen)) != PObject::EqualTo)
+          continue;
+        if (PFile::Access(adjustedDevice, PFile::ReadOnly)) 
+          return true;
+        PTRACE(1, "Unable to access file '" << adjustedDevice << "' for use as a video input device");
+        return false;
+      }
+      return false;
     }
 } PVideoInputDevice_YUVFile_descriptor;
 
@@ -218,10 +230,10 @@ PBoolean PVideoInputDevice_YUVFile::SetColourFormat(const PString & newFormat)
 PBoolean PVideoInputDevice_YUVFile::SetFrameRate(unsigned rate)
 {
   // if the file does not know what frame rate it is, then set it
-  if (file == NULL || (file->IsUnknownFrameSize() && !file->SetFrameRate(rate)))
+  if ((file != NULL) && (file->IsUnknownFrameSize() || !file->SetFrameRate(rate)))
     return PFalse;
 
-  return PVideoDevice::SetFrameRate(file->GetFrameRate());
+  return PVideoDevice::SetFrameRate(rate);
 }
 
 
