@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21002 $
- * $Author: rjongbloed $
- * $Date: 2008-09-16 04:08:58 +0000 (Tue, 16 Sep 2008) $
+ * $Revision: 21579 $
+ * $Author: csoutheren $
+ * $Date: 2008-11-20 05:31:36 +0000 (Thu, 20 Nov 2008) $
  */
 
 #include <ptlib.h>
@@ -39,16 +39,37 @@
 #include <ctype.h>
 #include <ptlib/pfactory.h>
 #include <ptlib/pprocess.h>
+#include <ptlib/svcproc.h>
 
 #ifdef _WIN32
 #include <ptlib/msos/ptlib/debstrm.h>
+#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
+namespace PWLibStupidLinkerHacks {
+  extern int loadVideoForWindowsStuff;
+};
+#endif
 #endif
 
-#ifdef __MACOSX__
-namespace PWLibStupidOSXHacks {
+#if defined(P_MINGW)
+namespace PWLibStupidLinkerHacks {
+  extern int loadFakeVideoStuff;
+#if defined(P_AUDIO)
+  extern int loadWindowsMultimediaStuff;
+#endif
+#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
+  extern int loadVideoForWindowsStuff;
+#endif
+#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
+  extern int loadDirectShowStuff;
+#endif
+};
+#endif
+     
+#ifdef P_MACOSX
+namespace PWLibStupidLinkerHacks {
+  extern int loadFakeVideoStuff;
   extern int loadShmVideoStuff;
   extern int loadCoreAudioStuff;
-  extern int loadFakeVideoStuff;
 };
 #endif
 
@@ -512,10 +533,11 @@ ostream & PTrace::End(ostream & paramStream)
   }
   else {
 #ifdef _WIN32_WCE
-    *info.stream << "\r\n";
-#else
-    *info.stream << '\n';
+    if (!PIsDescendant(info.stream, PDebugStream) && info.stream != &cerr)
+      *info.stream << "\r\n";
+    else
 #endif
+    *info.stream << '\n';
   }
   info.stream->flush();
 
@@ -1503,6 +1525,11 @@ PProcess::PProcess(const char * manuf, const char * name,
         executableFile = execFile;
     }
   }
+
+#if defined(P_VIDEO)  && defined(P_VFW_CAPTURE)
+  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
+#endif
+
 #endif // _WIN32
 
   if (productName.IsEmpty())
@@ -1514,20 +1541,40 @@ PProcess::PProcess(const char * manuf, const char * name,
 
   Construct();
 
-#ifdef __MACOSX__
-  
+#if defined(P_MINGW)
 #ifdef P_VIDEO
-  PWLibStupidOSXHacks::loadFakeVideoStuff = 1;
+  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
+#endif
+#if defined(P_AUDIO)
+  PWLibStupidLinkerHacks::loadWindowsMultimediaStuff = 1;
+#endif
+#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
+  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
+#endif
+#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
+  PWLibStupidLinkerHacks::loadDirectShowStuff = 1;
+#endif
+
+#endif
+
+
+#ifdef P_MACOSX
+
+#ifdef P_VIDEO
+
+  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
+  
 #ifdef P_SHM_VIDEO
-  PWLibStupidOSXHacks::loadShmVideoStuff = 1;
+  PWLibStupidLinkerHacks::loadShmVideoStuff = 1;
 #endif // P_SHM_VIDEO
-#endif // P_VIDEO
+
+#endif
   
 #ifdef P_AUDIO
-  PWLibStupidOSXHacks::loadCoreAudioStuff = 1;
+  PWLibStupidLinkerHacks::loadCoreAudioStuff = 1;
 #endif // P_AUDIO
   
-#endif // __MACOSX__
+#endif // P_MACOSX
 
   // create one instance of each class registered in the 
   // PProcessStartup abstract factory
@@ -2324,5 +2371,24 @@ PWriteWaitAndSignal::~PWriteWaitAndSignal()
   mutex.EndWrite();
 }
 
+/////////////////////////////////////////////////////////////////////////////
+
+PSystemLog::PSystemLog(Level level)   ///< only messages at this level or higher will be logged
+  : iostream(cout.rdbuf()) 
+{ 
+  logLevel = level; 
+  buffer.log = this; 
+  init(&buffer); 
+}
+
+PSystemLog::PSystemLog(const PSystemLog & other)
+  : PObject(other), iostream(cout.rdbuf()) 
+{
+}
+
+PSystemLog & PSystemLog::operator=(const PSystemLog &)
+{ 
+  return *this; 
+}
 
 // End Of File ///////////////////////////////////////////////////////////////
