@@ -23,9 +23,9 @@
  *
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Revision: 21176 $
+ * $Author: ms30002000 $
+ * $Date: 2008-09-27 20:34:32 +0000 (Sat, 27 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -39,8 +39,8 @@
 #include <ptlib/videoio.h>
 #include <ptclib/delaychan.h>
 
-#ifdef __MACOSX__
-namespace PWLibStupidOSXHacks {
+#if defined(P_MACOSX) || defined(P_MINGW)
+namespace PWLibStupidLinkerHacks {
   int loadFakeVideoStuff;
 };
 #endif
